@@ -2,11 +2,13 @@ from pyrf.config import SweepEntry, TriggerSettings, TriggerSettingsError
 from pyrf.connectors.blocking import PlainSocketConnector
 from pyrf.connectors.base import sync_async
 from pyrf.vrt import vrt_packet_reader, I_ONLY, IQ
-
+from pyrf.windows_util import get_broadcast_addresses
 from pyrf.units import M
 
 import struct
-
+import socket
+import select
+import platform
 
 DISCOVERY_UDP_PORT = 18331
 _DISCOVERY_QUERY_CODE = 0x93315555
@@ -47,32 +49,46 @@ class WSA5000_220Properties(object):
 
     ADC_DYNAMIC_RANGE = 72.5
     NOISEFLOOR_CALIBRATION = -10
-    CAPTURE_FREQ_RANGES = [(0, 50*M, I_ONLY), (100*M, 20000*M, IQ)]
+    CAPTURE_FREQ_RANGES = [(50*M, 20000*M, IQ)]
     SWEEP_FREQ_RANGE = (100*M, 20000*M)
 
-    RFE_MODES = ('ZIF', 'SH', 'HDR')
+    RFE_MODES = ('ZIF', 'SH', 'SHN', 'HDR', 'IQIN', 'DD')
 
-    FULL_BW = {'ZIF': 125*M, 'HDR': 162760, 'SH': 62500000}
-    USABLE_BW = {'ZIF': 100*M, 'HDR': 100000, 'SH': 40000000}
-    MIN_TUNABLE = {'ZIF': 50*M, 'HDR': 50*M, 'SH': 50*M}
-    MAX_TUNABLE = {'ZIF': 20000*M, 'HDR': 20000*M, 'SH': 20000*M}
-    MIN_DECIMATION = {'ZIF': 4, 'HDR': None, 'SH': 4}
-    MAX_DECIMATION = {'ZIF': 1024, 'HDR': None, 'SH': 4}
+    FULL_BW = {'ZIF': 125*M, 'HDR': 162760, 'SH': 62500000 , 'SHN': 62500000, 'IQIN': 625000000, 'DD': 625000000}
+    USABLE_BW = {'ZIF': 100*M, 'HDR': 100000, 'SH': 40000000,'SHN': 10000000, 'IQIN': 50000000, 'DD': 0}
+    MIN_TUNABLE = {'ZIF': 50*M, 'HDR': 50*M, 'SH': 50*M, 'SHN': 50*M,'IQIN': 0.1*M, 'DD': 0}
+    MAX_TUNABLE = {'ZIF': 20000*M, 'HDR': 20000*M, 'SH': 20000*M,'SHN': 20000*M, 'IQIN': 0.1*M, 'DD': 0}
+    MIN_DECIMATION = {'ZIF': 4, 'HDR': None, 'SH': 4, 'SHN': 4,'IQIN': None, 'DD': None}
+    MAX_DECIMATION = {'ZIF': 1024, 'HDR': None, 'SH': 4, 'SHN': 4,'IQIN': None, 'DD': None}
     DECIMATED_USABLE = 0.80
-    PASS_BAND_CENTER = {'ZIF': 0.5, 'HDR': 0.6, 'SH': 0.6}
+    PASS_BAND_CENTER = {'ZIF': 0.5, 'HDR': 0.6, 'SH': 0.56, 'SHN': 0.56,'IQIN': 0.5, 'DD': 0.5}
     DC_OFFSET_BW = 240000 # XXX: an educated guess
     TUNING_RESOLUTION = 100000
-    FSHIFT_AVAILABLE = {'ZIF': True, 'HDR': False, 'SH': True}
+    FSHIFT_AVAILABLE = {'ZIF': True, 'HDR': False, 'SH': True, 'SHN': True,'IQIN': False, 'DD': False}
     MAX_FSHIFT = {'ZIF': 62.5*M}
-    SWEEP_SETTINGS = ['fstart', 'fstop', 'fstep', 'fshift', 'decimation',
-        'attenuator', 'ifgain', 'spp', 'ppb', 'dwell_s', 'dwell_us',
+    SWEEP_SETTINGS = ['rfe_mode', 'fstart', 'fstop', 'fstep', 'fshift',
+        'decimation', 'attenuator', 'ifgain', 'spp', 'ppb',
+        'dwell_s', 'dwell_us',
         'trigtype', 'level_fstart', 'level_fstop', 'level_amplitude']
 
 
+class WSA5000_220_v2Properties(WSA5000_220Properties):
+    model = 'WSA5000-220 v2'
+    # v2 -> hardware revision without SHN mode
+
+    RFE_MODES = ('ZIF', 'SH', 'HDR', 'IQIN', 'DD')
+
+
 class WSA5000_208Properties(WSA5000_220Properties):
     model = 'WSA5000-208'
+    # 208 -> limited to 8GHz
 
-    MAX_TUNABLE = {'ZIF': 8000*M, 'HDR': 8000*M, 'SH': 8000*M}
+    MAX_TUNABLE = dict((mode, min(8000*M, f))
+        for mode, f in WSA5000_220Properties.MAX_TUNABLE.iteritems())
+
+
+class WSA5000_208_v2Properties(WSA5000_220_v2Properties, WSA5000_208Properties):
+    model = 'WSA5000-208 v2'
 
 
 class WSA(object):
@@ -123,6 +139,10 @@ class WSA(object):
         device_id = (yield self.scpiget(":*idn?"))
         if device_id.startswith('ThinkRF,WSA4000'):
             self.properties = WSA4000Properties
+        elif device_id.startswith('ThinkRF,WSA5000-220 v2'):
+            self.properties = WSA5000_220_v2Properties
+        elif device_id.startswith('ThinkRF,WSA5000-208 v2'):
+            self.properties = WSA5000_208_v2Properties
         elif device_id.startswith('ThinkRF,WSA5000-208'):
             self.properties = WSA5000_208Properties
         else:
@@ -192,6 +212,39 @@ class WSA(object):
 
         yield mode
 
+    @sync_async
+    def iq_output_path(self, path=None):
+        """
+        This command sets or queries the WSA's current IQ path
+
+        :param path: 'DIGITIZER', 'CONNECTOR', or None to query
+        :returns: the current IQ path
+        """
+
+        if path is None:
+            buf = yield self.scpiget(":OUTPUT:IQ:MODE?")
+            path = buf.strip()
+        else:
+            self.scpiset(":OUTPUT:IQ:MODE %s" % path)
+        yield path
+
+    @sync_async
+    def pll_reference(self, src=None):
+        """
+        This command sets or queries the WSA's PLL reference source
+
+        :param src: 'INT', 'EXT', or None to query
+        :returns: the current PLL reference source
+        """
+
+        if src is None:
+            buf = yield self.scpiget(":SOURCE:REFERENCE:PLL?")
+            src = buf.strip()
+        else:
+            assert src in ('INT', 'EXT')
+            self.scpiset(":SOURCE:REFERENCE:PLL %s" % src)
+        yield src
+
     @sync_async
     def freq(self, freq=None):
         """
@@ -541,6 +594,8 @@ class WSA(object):
         :type entry: pyrf.config.SweepEntry
         """
         self.scpiset(":sweep:entry:new")
+        if 'rfe_mode' in self.properties.SWEEP_SETTINGS:
+            self.scpiset(":sweep:entry:mode %s" % (entry.rfe_mode))
         self.scpiset(":sweep:entry:freq:center %d, %d" % (entry.fstart, entry.fstop))
         self.scpiset(":sweep:entry:freq:step %d" % (entry.fstep))
         self.scpiset(":sweep:entry:freq:shift %d" % (entry.fshift))
@@ -710,6 +765,8 @@ class WSA(object):
             'trigger': self.trigger,
             'attenuator': self.attenuator,
             'rfe_mode': self.rfe_mode,
+            'iq_output_path': self.iq_output_path,
+            'pll_reference': self.pll_reference,
             }
 
         for k, v in settings.iteritems():
@@ -718,7 +775,10 @@ class WSA(object):
 
 def parse_discovery_response(response):
     """
-    Return (model, serial, firmware version) based on a discovery
+    This function parses the WSA's raw discovery response
+
+    :param response: The WSA's raw response to a discovery query
+    :returns: Return (model, serial, firmware version) based on a discovery
     response message
     """
     RESPONSE_HEADER_FORMAT = '>II'
@@ -731,6 +791,45 @@ def parse_discovery_response(response):
     return tuple(v.rstrip('\0') for v in struct.unpack(WSA5000_FORMAT,
         response[8:]))
 
+def discover_wsa(wait_time=0.125):
+    """
+    This function returns a list that contains all of the WSA's available
+    on the local network
+
+    :param wait_time: The total time to wait for responses in seconds
+    :returns: Return a list of dicts (MODEL, SERIAL, FIRMWARE, IP) of all the WSA's
+    available on the local network
+    """
+
+    cs = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+    cs.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
+    cs.setblocking(0)
+
+    wsa_list = []
+
+    if platform.system() == 'Windows':
+        destinations = get_broadcast_addresses()
+    else:
+        destinations = ['<broadcast>']
+
+    for d in destinations:
+        # send query command to WSA
+        query_struct = DISCOVERY_QUERY
+        cs.sendto(query_struct, (d, DISCOVERY_UDP_PORT))
+
+    while True:
+        ready, _, _ = select.select([cs], [], [], wait_time)
+        if not ready:
+            break
+        data, (host, port) = cs.recvfrom(1024)
+
+        model, serial, firmware = parse_discovery_response(data)
+
+        wsa_list.append({"MODEL": model,
+                        "SERIAL": serial,
+                        "FIRMWARE": firmware,
+                        "HOST": host})
+    return  wsa_list
 
 # for backwards compatibility
 WSA4000 = WSA
