@@ -13,6 +13,7 @@ import sys
 from PySide import QtGui, QtCore
 import numpy as np
 import math
+
 from contextlib import contextmanager
 from pkg_resources import parse_version
 
@@ -29,12 +30,14 @@ from pyrf.numpy_util import compute_fft
 from pyrf.devices.thinkrf import WSA
 from pyrf.vrt import (I_ONLY, VRT_IFDATA_I14Q14, VRT_IFDATA_I14,
     VRT_IFDATA_I24, VRT_IFDATA_PSD8)
-    
+
 from util import find_max_index, find_nearest_index
 from util import hotkey_util, update_marker_traces
 import control_util as cu
 from plot_widget import Plot
 from device_widget import DeviceControlsWidget
+from discovery_widget import DiscoveryWidget
+
 RBW_VALUES = [976.562, 488.281, 244.141, 122.070, 61.035, 30.518, 15.259, 7.62939, 3.815]
 
 HDR_RBW_VALUES = [1271.56, 635.78, 317.890, 158.94, 79.475, 39.736, 19.868, 9.934]
@@ -62,15 +65,14 @@ class MainWindow(QtGui.QMainWindow):
         WINDOW_HEIGHT = screen.height() * 0.6
         self.resize(WINDOW_WIDTH,WINDOW_HEIGHT)
         self.initUI(output_file)
-        self.show()
 
     def initUI(self, output_file):
         name = None
         if len(sys.argv) > 1:
             name = sys.argv[1]
-        self.mainPanel = MainPanel(output_file)
+        self.mainPanel = MainPanel(self,output_file)
         openAction = QtGui.QAction('&Open Device', self)
-        openAction.triggered.connect( self.mainPanel.open_device_dialog)
+        openAction.triggered.connect( self.open_device_dialog)
         exitAction = QtGui.QAction('&Exit', self)
         exitAction.setShortcut('Ctrl+Q')
         exitAction.triggered.connect(self.close)
@@ -80,12 +82,17 @@ class MainWindow(QtGui.QMainWindow):
         fileMenu.addAction(exitAction)
         self.setWindowTitle('Spectrum Analyzer')
         self.setCentralWidget(self.mainPanel)
-        self.mainPanel.show()
         if name:
-            self.mainPanel.open_device(name)
+            self.mainPanel.open_device(name, True)
         else:
-            self.mainPanel.open_device_dialog()
-    
+            self.open_device_dialog()
+
+    def open_device_dialog(self):
+        self.discovery_widget = DiscoveryWidget(
+            open_device_callback=self.mainPanel.open_device,
+            name="Open Device")
+        self.discovery_widget.show()
+
     def closeEvent(self, event):
         if self.mainPanel.dut:
             self.mainPanel.dut.abort()
@@ -94,12 +101,12 @@ class MainWindow(QtGui.QMainWindow):
         event.accept()
         self.mainPanel._reactor.stop()
 
-
 class MainPanel(QtGui.QWidget):
     """
     The spectrum view and controls
     """
-    def __init__(self, output_file):
+    def __init__(self, main_window,output_file):
+        self._main_window = main_window
         self.ref_level = 0
         self.dut = None
         self.control_widgets = []
@@ -123,24 +130,18 @@ class MainPanel(QtGui.QWidget):
         from twisted.internet import reactor
         return reactor
 
-    def open_device_dialog(self):
-        name, ok = QtGui.QInputDialog.getText(self, 'Open Device',
-            'Enter a hostname or IP address:')
-        while True:
-            if not ok:
-                return
-            try:
-                self.open_device(name)
-                break
-            except socket.error:
-                name, ok = QtGui.QInputDialog.getText(self, 'Open Device',
-                    'Connection Failed, please try again\n\n'
-                    'Enter a hostname or IP address:')
-
     @inlineCallbacks
-    def open_device(self, name):
+    def open_device(self, name, ok):
+        if not ok:
+            self._main_window.show()
+            return
+
+        if self.dut:
+            self.dut.disconnect()
+        self._main_window.show()
         dut = WSA(connector=TwistedConnector(self._reactor))
         yield dut.connect(name)
+
         if hasattr(dut.properties, 'MINIMUM_FW_VERSION') and parse_version(
                 dut.fw_version) < parse_version(dut.properties.MINIMUM_FW_VERSION):
             too_old = QtGui.QMessageBox()
@@ -151,6 +152,7 @@ class MainPanel(QtGui.QWidget):
             too_old.exec_()
         if self._output_file:
             dut.set_capture_output(self._output_file)
+
         self.dut = dut
         self.plot_state = gui_config.PlotState(dut.properties)
         self.dut_prop = self.dut.properties
@@ -172,6 +174,7 @@ class MainPanel(QtGui.QWidget):
         device_set.pop('freq')
         device_set.pop('decimation')
         device_set.pop('fshift')
+        device_set.pop('iq_output_path')
         self.sweep_dut.capture_power_spectrum(self.plot_state.fstart,
                                               self.plot_state.fstop,
                                               self.plot_state.rbw,
@@ -186,6 +189,7 @@ class MainPanel(QtGui.QWidget):
         # store usable bins before next call to capture_time_domain
         self.usable_bins = list(self.cap_dut.usable_bins)
         self.sweep_segments = None
+
         if not self.plot_state.block_mode:
             self.read_sweep()
             return
@@ -196,8 +200,6 @@ class MainPanel(QtGui.QWidget):
         self.pow_data = compute_fft(self.dut, data['data_pkt'], data['context_pkt'], ref = self.ref_level)
         self.raw_data = data['data_pkt']
 
-
-
         self.update_plot()
 
     def receive_sweep(self, fstart, fstop, data):
@@ -243,9 +245,9 @@ class MainPanel(QtGui.QWidget):
 
         # add plot widget
         plot_width = 8
-        
+
         grid.addWidget(self._plot_layout(),0,0,13,plot_width)
-        
+
         self.marker_labels = []
         marker_label, delta_label, diff_label = self._marker_labels()
         self.marker_labels.append(marker_label)
@@ -253,7 +255,7 @@ class MainPanel(QtGui.QWidget):
         grid.addWidget(marker_label, 0, 1, 1, 2)
         grid.addWidget(delta_label, 0, 3, 1, 2)
         grid.addWidget(diff_label , 0, 5, 1, 2)
- 
+
         y = 0
         x = plot_width
 
@@ -289,14 +291,14 @@ class MainPanel(QtGui.QWidget):
 
     def _trace_controls(self):
         trace_group = QtGui.QGroupBox("Traces")
-  
+
         self._trace_group = trace_group
-        
+
         trace_controls_layout = QtGui.QVBoxLayout()
-        
+
         # first row will contain the tabs
         first_row = QtGui.QHBoxLayout()
-        
+
         # add tabs for each trace
         trace_tab = QtGui.QTabBar()
         count = 0
@@ -309,13 +311,13 @@ class MainPanel(QtGui.QWidget):
             icon = QtGui.QIcon(pixmap)
             trace_tab.setTabIcon(count,icon)
             count += 1
-        
+
         self._trace_tab = trace_tab
         trace_tab.currentChanged.connect(lambda: cu._trace_tab_change(self))
-            
+
         self.control_widgets.append(self._trace_tab)
         first_row.addWidget(trace_tab)
-        
+
         # second row contains the tab attributes
         second_row = QtGui.QHBoxLayout()
         max_hold, min_hold, write, store, blank  = self._trace_items()
@@ -328,9 +330,9 @@ class MainPanel(QtGui.QWidget):
         trace_controls_layout.addLayout(second_row) 
         trace_group.setLayout(trace_controls_layout)
         return trace_group
-        
+
     def _trace_items(self):
-    
+
         trace_attr = {}
         store = QtGui.QCheckBox('Store')
         store.clicked.connect(lambda: cu._store_trace(self))
@@ -340,19 +342,19 @@ class MainPanel(QtGui.QWidget):
         max_hold = QtGui.QRadioButton('Max Hold')
         max_hold.clicked.connect(lambda: cu._max_hold(self))
         trace_attr['max_hold'] = max_hold
-        
+
         min_hold = QtGui.QRadioButton('Min Hold')
         min_hold.clicked.connect(lambda: cu._min_hold(self))
         trace_attr['min_hold'] = min_hold
-        
+
         write = QtGui.QRadioButton('Write')
         write.clicked.connect(lambda: cu._trace_write(self))
         trace_attr['write'] = write
-         
+
         blank = QtGui.QRadioButton('Blank')
         blank.clicked.connect(lambda: cu._blank_trace(self))
         trace_attr['blank'] = blank
-                
+
         self._trace_attr = trace_attr
         self._trace_attr['write'].click()
         return max_hold, min_hold, write, store, blank
@@ -368,11 +370,13 @@ class MainPanel(QtGui.QWidget):
         def new_antenna():
             self.plot_state.dev_set['antenna'] = (int(self._dev_group._antenna_box.currentText().split()[-1]))
             self.cap_dut.configure_device(self.plot_state.dev_set)
+        
         def new_dec():
             self.plot_state.dev_set['decimation'] = int(
                 self._dev_group._dec_box.currentText().split(' ')[-1])
             self.cap_dut.configure_device(self.plot_state.dev_set)
-        
+            self.update_freq()
+
         def new_freq_shift():
             rfe_mode = 'ZIF'
             prop = self.dut.properties
@@ -386,19 +390,39 @@ class MainPanel(QtGui.QWidget):
                 self._dev_group._freq_shift_edit.setText(str(self.plot_state.dev_set['fshift'] / M))
                 return
             self.cap_dut.configure_device(self.plot_state.dev_set)
-        
+
         def new_gain():
             self.plot_state.dev_set['gain'] = self._dev_group._gain_box.currentText().split()[-1].lower().encode('ascii')
             self.cap_dut.configure_device(self.plot_state.dev_set)
-        
+
         def new_ifgain():
             self.plot_state.dev_set['ifgain'] = self._dev_group._ifgain_box.value()
             self.cap_dut.configure_device(self.plot_state.dev_set)
-        
+
         def new_attenuator():
             self.plot_state.dev_set['attenuator'] = self._dev_group._attenuator_box.isChecked()
             self.cap_dut.configure_device(self.plot_state.dev_set)
-        
+
+        def new_pll_reference():
+            if 'INTERNAL' in str(self._dev_group._pll_box.currentText()):
+                src = 'INT'
+            else:
+                src = 'EXT'
+            self.plot_state.dev_set['pll_reference'] = src
+            self.cap_dut.configure_device(self.plot_state.dev_set)
+
+        def new_iq_path():
+            self.plot_state.dev_set['iq_output_path'] = str(self._dev_group._iq_output_box.currentText().split()[-1])
+            
+            if self.plot_state.dev_set['iq_output_path'] == 'CONNECTOR':
+                # disable unwanted controls
+                cu._external_digitizer_mode(self)
+            else:
+                cu._internal_digitizer_mode(self)
+                self.cap_dut.configure_device(self.plot_state.dev_set)
+                self.read_block()
+            self.cap_dut.configure_device(self.plot_state.dev_set)
+
         def new_input_mode():
             m = self._dev_group._mode.currentText()
             if not m:
@@ -419,6 +443,9 @@ class MainPanel(QtGui.QWidget):
             self.plot_state.dev_set['rfe_mode'] = str(m)
             cu._update_rbw_values(self)
             self.plot_state.bandwidth = self.dut_prop.FULL_BW[m]
+            if self.plot_state.dev_set['rfe_mode'] == 'IQIN':
+                self._freq_edit.setText(str(self.dut_prop.MIN_TUNABLE[self.plot_state.dev_set['rfe_mode']]/M))
+            
             self.plot_state.update_freq_set(
                 fcenter=float(self._freq_edit.text()) * M)
             cu._center_plot_view(self)
@@ -440,7 +467,8 @@ class MainPanel(QtGui.QWidget):
         self._dev_group._ifgain_box.valueChanged.connect(new_ifgain)
         self._dev_group._attenuator_box.clicked.connect(new_attenuator)
         self._dev_group._mode.currentIndexChanged.connect(new_input_mode)
-
+        self._dev_group._iq_output_box.currentIndexChanged.connect(new_iq_path)
+        self._dev_group._pll_box.currentIndexChanged.connect(new_pll_reference)
     
     def _trigger_control(self):
         trigger = QtGui.QCheckBox("Trigger")
@@ -463,52 +491,60 @@ class MainPanel(QtGui.QWidget):
     def _freq_controls(self):
         freq_group = QtGui.QGroupBox("Frequency Control")
         self._freq_group = freq_group
-        
+
         freq_layout = QtGui.QVBoxLayout()
-        
+
         fstart_hbox = QtGui.QHBoxLayout()
         fstart_bt, fstart_txt = self._fstart_controls()
         fstart_hbox.addWidget(fstart_bt)
         fstart_hbox.addWidget(fstart_txt)
-        
+        fstart_hbox.addWidget(QtGui.QLabel('MHz'))
+        self._fstart_hbox = fstart_hbox
+
         cfreq_hbox = QtGui.QHBoxLayout()
         cfreq_bt, cfreq_txt = self._center_freq()
         cfreq_hbox.addWidget(cfreq_bt)
         cfreq_hbox.addWidget(cfreq_txt)
         cfreq_hbox.addWidget(QtGui.QLabel('MHz'))
-        
+        self._cfreq_hbox = cfreq_hbox
+
         bw_hbox = QtGui.QHBoxLayout()
         bw_bt, bw_txt = self._bw_controls()
         bw_hbox.addWidget(bw_bt)
         bw_hbox.addWidget(bw_txt)
+        bw_hbox.addWidget(QtGui.QLabel('MHz'))
+        self._bw_hbox = bw_hbox
 
-        
         fstop_hbox = QtGui.QHBoxLayout()
         fstop_bt, fstop_txt = self._fstop_controls()
         fstop_hbox.addWidget(fstop_bt)
         fstop_hbox.addWidget(fstop_txt)
+        fstop_hbox.addWidget(QtGui.QLabel('MHz'))
+        self._fstop_hbox = fstop_hbox
 
-        
         freq_inc_hbox = QtGui.QHBoxLayout()
         freq_inc_steps, freq_inc_plus, freq_inc_minus = self._freq_incr()
         freq_inc_hbox.addWidget(freq_inc_minus)
         freq_inc_hbox.addWidget(freq_inc_steps)
         freq_inc_hbox.addWidget(freq_inc_plus)
-        
+        self._freq_inc_hbox = freq_inc_hbox
+
         rbw_hbox = QtGui.QHBoxLayout()
         rbw = self._rbw_controls()
         rbw_hbox.addWidget(QtGui.QLabel('Resolution Bandwidth:'))
         rbw_hbox.addWidget(rbw)
+        self._rbw_hbox = rbw_hbox
         
-        freq_layout.addLayout(fstart_hbox)
-        freq_layout.addLayout(cfreq_hbox)
-        freq_layout.addLayout(bw_hbox)
-        freq_layout.addLayout(fstop_hbox)
-        freq_layout.addLayout(freq_inc_hbox)
-        freq_layout.addLayout(rbw_hbox)
+        freq_layout.addLayout(self._fstart_hbox)
+        freq_layout.addLayout(self._cfreq_hbox)
+        freq_layout.addLayout(self._bw_hbox)
+        freq_layout.addLayout(self._fstop_hbox)
+        freq_layout.addLayout(self._freq_inc_hbox)
+        freq_layout.addLayout(self._rbw_hbox)
         freq_group.setLayout(freq_layout)
-        
+        self._freq_layout = freq_layout
         return freq_group
+
     def _center_freq(self):
         cfreq = QtGui.QPushButton('Center')
         cfreq.setToolTip("[2]\nTune the center frequency") 
@@ -522,10 +558,10 @@ class MainPanel(QtGui.QWidget):
             cu._select_center_freq(self)
             self.update_freq()
             self.update_freq_edit()
-        
+
         freq_edit.returnPressed.connect(lambda: freq_change())
         return cfreq, freq_edit
-    
+
     def _freq_incr(self):
         steps = QtGui.QComboBox(self)
         steps.addItem("Adjust: 1 MHz")
@@ -557,8 +593,7 @@ class MainPanel(QtGui.QWidget):
         self.control_widgets.append(self._freq_minus)
         self.control_widgets.append(self._fstep_box)
         return  steps, freq_plus, freq_minus
-    
-    
+
     def _bw_controls(self):
         bw = QtGui.QPushButton('Span')
         bw.setToolTip("[3]\nChange the bandwidth of the current plot")
@@ -574,7 +609,7 @@ class MainPanel(QtGui.QWidget):
         self.control_widgets.append(self._bw_edit)
         self.control_widgets.append(self._bw)
         return bw, bw_edit
-    
+
     def _fstart_controls(self):
         fstart = QtGui.QPushButton('Start')
         fstart.setToolTip("[1]\nTune the start frequency")
@@ -586,13 +621,13 @@ class MainPanel(QtGui.QWidget):
             cu._select_fstart(self)
             self.update_freq()
             self.update_freq_edit()
-            
+
         freq.returnPressed.connect(lambda: freq_change())
         self._fstart_edit = freq
         self.control_widgets.append(self._fstart)
         self.control_widgets.append(self._fstart_edit)
         return fstart, freq
-        
+
     def _fstop_controls(self):
         fstop = QtGui.QPushButton('Stop')
         fstop.setToolTip("[4]Tune the stop frequency") 
@@ -609,7 +644,7 @@ class MainPanel(QtGui.QWidget):
         self.control_widgets.append(self._fstop)
         self.control_widgets.append(self._fstop_edit)
         return fstop, freq
-           
+
     def _rbw_controls(self):
         rbw = QtGui.QComboBox(self)
         rbw.setToolTip("Change the RBW of the FFT plot")
@@ -617,7 +652,7 @@ class MainPanel(QtGui.QWidget):
         self._hdr_points_values = HDR_RBW_VALUES
         self._rbw_box = rbw
         rbw.addItems([str(p) + ' KHz' for p in self._points_values])
-        
+
         def new_rbw():
         
             if not self.plot_state.dev_set['rfe_mode'] == 'HDR':    
@@ -641,7 +676,6 @@ class MainPanel(QtGui.QWidget):
             delta = 0    
         try:
             if self.plot_state.freq_sel == 'CENT':
-                
                 f = (float(self._freq_edit.text()) + delta) * M
                 if f > max_tunable or f < min_tunable:
                     return
@@ -670,14 +704,14 @@ class MainPanel(QtGui.QWidget):
                     trace.data = self.pow_data
                 except AttributeError:
                     break
-					
+
         except ValueError:
             return
         if self.plot_state.trig:
             freq_region = self._plot.freqtrig_lines.getRegion()
             if (freq_region[0] < self.plot_state.fstart and freq_region[1] < self.plot_state.fstart) or (freq_region[0] > self.plot_state.fstop and freq_region[1] > self.plot_state.fstop):
                 self._plot.freqtrig_lines.setRegion([self.plot_state.fstart,self.plot_state. fstop]) 
-    
+
     def update_freq_edit(self):
         self._fstop_edit.setText("%0.1f" % (self.plot_state.fstop/ 1e6))
         self._fstart_edit.setText("%0.1f" % (self.plot_state.fstart/ 1e6))
